%% TEAM: UN-KIT-SWITCHING BATTERY
% Cooperation of Universidad Nacional de Colombia (UN), Karlsruhe Institute
% of Technology (KIT) and SWITCHING BATTERY company
%% TEAM MEMBERS: 
% Sergio Rivera, srriverar@unal.edu.co, professor at UN
% Sebastian Krumscheid, sebastian.krumscheid@kit.edu, professor at KIT
% Kannappan Chettiar, kc@switchingbattery.com, CEO SWITCHING BATTERY

%% ENSEMBLED ALGORITMH: MVMO-RCEDUMDA-HyDE

%% 
% THIS SCRIPT IS BASED ON PUBLIC PREVIOUS DEVELOPMENTS OF MVMO (Mean-variance
% mapping optimization AND RCEDUMDA (Ring Cellular Encode-Decode UMDA) AND
% HyDE (Hybrid-adaptive differential evolution)

clc;clear all;close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GECAD GECCO and WCCI 2022 Competition: Evolutionary Computation in the Energy Domain: Risk-based Energy Scheduling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('functions') %Necessary functions to run the algorithms (encrypted)

tTotalTime=tic; % lets track total computational time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Select testbed
Select_testbed=2;
%Testbed 1: Risk-based energy scheduling
%Testbed 2: Transmission expansion planning

DB=Select_testbed;
% 1: Case study testbed 1
% 2: Case study testbed 2

Select_algorithm=2;
%1: HyDE algorithm (test algorithm)
%2: Your algorithm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load MH parameters (e.g., get MH parameters from DEparameters.m file)
switch Select_algorithm
       case 1
          addpath('HyDE')
          algorithm='HyDE-Algorithm'; %'The participants should include their algorithm here'
          DEparameters %Function defined by the participant
          No_solutions=deParameters.I_NP; %Notice that some algorithms are limited to one individual
       case 2
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          %% ALGORITMH: MVMO-RCEDUMDA
          addpath('MVMO')
          algorithm='MVMO-Algorithm'; %'The participants should include their algorithm here'
          DEparameters %Function defined by the participant
          No_solutions=deParameters.I_NP; %Notice that some algorithms are limited to one individual
          %%%%%%%%%%%%%%%%%%%%%% Your algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%%

    otherwise
          fprintf(1,'No algorithm selected\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load Data base 
[caseStudyData, DB_name]=callDatabase(DB);
noRuns=20; %this can be changed but final results should be based on 20 trials

%% Label of the algorithm and the case study
Tag.algorithm=algorithm;
Tag.DB=DB_name;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set other parameters
otherParameters =setOtherParameters(caseStudyData, No_solutions, Select_testbed);
otherParameters.one = setOtherParameters(caseStudyData,1, Select_testbed);
%% Set lower/upper bounds of variables 
[lowerB,upperB] = setVariablesBounds(caseStudyData,otherParameters, Select_testbed);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Call the MH for optimizationclear 
ResDB=struc([]);
for iRuns=1:noRuns %Number of trails
    tOpt=tic;
    rand('state',sum(noRuns*100*clock))% ensure stochastic indpt trials
    otherParameters.iRuns=iRuns;      

    switch Select_algorithm
        case 1
            [ResDB(iRuns).Fit_and_p, ...
            ResDB(iRuns).sol, ...
            ResDB(iRuns).fitVector]= ...                    
            HyDE(deParameters,caseStudyData,otherParameters,lowerB,upperB);
        case 2     
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% ALGORITMH: MVMO-RCEDUMDA
            % first ensembled, first iterations
            addpath('MVMO')
            algorithm='MVMO-Algorithm'; %'The participants should include their algorithm here'
            DEparameters %Function defined by the participant
            No_solutions=1;%deParameters.I_NP; %Notice that some algorithms are limited to one individual
                      
            otherParameters =setOtherParameters(caseStudyData, No_solutions, Select_testbed);
            otherParameters.one = setOtherParameters(caseStudyData,1, Select_testbed);
            otherParameters.iRuns=iRuns;  
                      
            [ResDB(iRuns).Fit_and_p, ...
             ResDB(iRuns).sol, ...
             ResDB(iRuns).fitVector]= ...                    
             MVMO(deParameters,caseStudyData,otherParameters,lowerB,upperB, Select_testbed);%,ResDB(iRuns).solution);  
                    
                      % second ensembled, second iterations  
                      addpath('RCEDUMDAalg2022')
                      algorithm='RCEDUMDA'; %'The participants should include their algorithm here'
                      RCEDUMDAparameters %Function defined by the participant
                      %No_solutions=vRCEDUMDAparameters.I_NP; %Notice that some algorithms are limited to one individual      
                      No_solutions2=1;
                                
                      otherParameters =setOtherParameters(caseStudyData, No_solutions2, Select_testbed);
                      otherParameters.one = setOtherParameters(caseStudyData,1, Select_testbed);
                      otherParameters.iRuns=iRuns; 
          
                      [ResDB(iRuns).Fit_and_p, ...
                      ResDB(iRuns).sol, ...
                      ResDB(iRuns).fitVector]= ...
                      RCEDUMDA(vRCEDUMDAparameters,caseStudyData,otherParameters,lowerB,upperB,ResDB(iRuns), Select_testbed);
                  
             % third ensembled, third iterations
             addpath('HyDE')
             algorithm='HyDE-Algorithm'; %'The participants should include their algorithm here'
             DEparameters %Function defined by the participant
             No_solutions=deParameters.I_NP; %Notice that some algorithms are limited to one individual     
             
            otherParameters =setOtherParameters(caseStudyData, No_solutions, Select_testbed);
            otherParameters.one = setOtherParameters(caseStudyData,1, Select_testbed);
            otherParameters.iRuns=iRuns; 
            
            [ResDB(iRuns).Fit_and_p, ...
            ResDB(iRuns).sol, ...
            ResDB(iRuns).fitVector]= ...                    
            HyDE(deParameters,caseStudyData,otherParameters,lowerB,upperB,ResDB(iRuns).sol, Select_testbed);
                  
          %%%%%%%%%%%%%%%%%%%%%% Your algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%%

    end 

        ResDB(iRuns).tOpt=toc(tOpt); % time of each trial
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% Save the results and stats
        Save_results

end

%% End of MH Optimization
for j=1:noRuns
    lower_violations=ResDB(j).sol<lowerB;
    upper_violations=ResDB(j).sol>upperB;
    many_low(j)=sum(lower_violations);
    many_up(j)=sum(upper_violations);
    fprintf('lower_violations : %d .\n', many_low(j));
    fprintf('upper_violations : %d .\n', many_up(j));
end

tTotalTime=toc(tTotalTime); %Total time

